package net.sf.doolin.context;

import net.sf.jstring.LocalizableException;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import java.util.Map;

/**
 * Abstract application based on a Spring application context.
 * 
 * @author Damien Coraboeuf
 * 
 */
public abstract class AbstractApplication implements Application,
		ApplicationContextAware {

	private ApplicationContext applicationContext;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setApplicationContext(ApplicationContext ctx)
			throws BeansException {
		this.applicationContext = ctx;
	}

	/**
	 * Gets a bean from the application, using a class as an identifier.
	 * 
	 * @param <T>
	 *            Type of the bean
	 * 
	 * @param idClass
	 *            Class as an ID
	 * @return Bean
	 */
	public <T> T getBean(Class<T> idClass) {
		if (isDefined(idClass.getName())) {
			@SuppressWarnings("unchecked")
			T bean = (T) getBean(idClass.getName());
			return bean;
		} else {
			@SuppressWarnings("unchecked")
			Map<String, T> map = applicationContext.getBeansOfType(idClass);
			if (map != null && !map.isEmpty()) {
				if (map.size() > 1) {
					throw new LocalizableException(
							"AbstractApplication.GetBean.MultipleTypes",
							idClass.getName());
				} else {
					return map.values().iterator().next();
				}
			} else {
				throw new LocalizableException("AbstractApplication.GetBean.NoType",
						idClass.getName());
			}
		}
	}

	/**
	 * Checks if a bean is defined in the application, using a class as an
	 * identifier.
	 * 
	 * @param idClass
	 *            Class as an ID
	 * @return <code>true</code> if the bean is defined
	 */
	public boolean isDefined(Class<?> idClass) {
		if (isDefined(idClass.getName())) {
			return true;
		} else {
			@SuppressWarnings("unchecked")
			Map<String, ?> map = applicationContext.getBeansOfType(idClass);
			return map != null && map.size() == 1;
		}
	}

	/**
	 * Checks if a bean is defined in the application
	 * 
	 * @param id
	 *            Bean ID
	 * @return <code>true</code> if the bean is defined
	 */
	public boolean isDefined(String id) {
		return applicationContext.containsBean(id);
	}

	/**
	 * Gets a bean from the application.
	 * 
	 * @param id
	 *            Bean ID
	 * @return Bean
	 */
	public Object getBean(String id) {
		return applicationContext.getBean(id);
	}

}
