package net.sf.doolin.context;

import net.sf.doolin.context.util.ContextUtils;
import net.sf.jstring.JStrings;
import net.sf.jstring.LocalizableException;
import org.springframework.context.ApplicationContext;

/**
 * Application.
 * 
 * @author Damien Coraboeuf
 * 
 */
public abstract class ApplicationLauncher {

	private static final String MESSAGE_APPLICATION_LAUNCHER_APPLICATION_NOT_FOUND = "ApplicationLauncher.ApplicationNotFound";

	static {
		JStrings.add("net.sf.doolin.context.ContextBundle");
	}

	/**
	 * Default bean name for the application.
	 */
	public static final String BEAN_APPLICATION = "Application";

	private ApplicationLauncher() {
	}

	/**
	 * Launches an application from a set of Spring contexts. The application is
	 * defined using the bean ID "Application".
	 * 
	 * @param contextPaths
	 *            List of Spring contexts.
	 * @return Launched application
	 */
	public static Application launch(String... contextPaths) {
		return launchNamed(BEAN_APPLICATION, contextPaths);
	}

	/**
	 * Launches an application from a set of Spring contexts. The application is
	 * defined using the bean ID "Application".
	 * 
	 * @param contextPaths
	 *            List of Spring contexts.
	 * @param arguments
	 *            Arguments for the launch
	 * @return Launched application
	 */
	public static Application launchWithArgs(String[] contextPaths,
			String[] arguments) {
		return launchNamedWithArgs(BEAN_APPLICATION, contextPaths, arguments);
	}

	/**
	 * Launches an application from a set of Spring contexts. The application is
	 * defined using the bean ID "Application".
	 * 
	 * @param id
	 *            Bean ID for the application
	 * @param contextPaths
	 *            List of Spring contexts.
	 * @return Launched application
	 */
	public static Application launchNamed(String id, String... contextPaths) {
		return launchNamedWithArgs(id, contextPaths, null);
	}

	/**
	 * Launches an application from a set of Spring contexts. The application is
	 * defined using the bean ID "Application".
	 * 
	 * @param id
	 *            Bean ID for the application
	 * @param contextPaths
	 *            List of Spring contexts.
	 * @param arguments
	 *            Arguments for the launch
	 * @return Launched application
	 */
	public static Application launchNamedWithArgs(String id,
			String[] contextPaths, String[] arguments) {
		// Loads the context
		ApplicationContext applicationContext = ContextUtils.load(contextPaths);
		// Gets the application
		if (applicationContext.containsBean(id)) {
			Application application = (Application) applicationContext
					.getBean(id);
			// Starts the application
			application.start(arguments);
			// OK
			return application;
		} else {
			throw new LocalizableException(
					MESSAGE_APPLICATION_LAUNCHER_APPLICATION_NOT_FOUND, id,
					contextPaths);
		}
	}

}
