package net.sf.doolin.context.spring;

import java.util.List;
import java.util.Map;

import net.sf.doolin.util.xml.DOMUtils;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.AbstractSingleBeanDefinitionParser;
import org.springframework.beans.factory.xml.BeanDefinitionParserDelegate;
import org.springframework.beans.factory.xml.ParserContext;
import org.w3c.dom.Element;

/**
 * Utility parser for beans.
 * 
 * @author Damien Coraboeuf
 */
public abstract class AbstractDefinitionParser extends
		AbstractSingleBeanDefinitionParser {

	/**
	 * Delegates the parsing of the current element to a child element.
	 * 
	 * @param element
	 *            Current element
	 * @param ns
	 *            Namespace URI for the child element
	 * @param childName
	 *            Name of the child element
	 * @param parser
	 *            Parser to apply
	 * @param context
	 *            Parsing context
	 * @param builder
	 *            Current builder
	 */
	protected void delegate(Element element, String ns, String childName,
			AbstractDefinitionParser parser, ParserContext context,
			BeanDefinitionBuilder builder) {
		Element eChild = DOMUtils.getElement(element, ns, childName);
		if (eChild != null) {
			parser.doParse(eChild, context, builder);
		}
	}

	/**
	 * Changes scope to public.
	 * 
	 * {@inheritDoc}
	 */
	@Override
	public void doParse(Element element, ParserContext parserContext,
			BeanDefinitionBuilder builder) {
		super.doParse(element, parserContext, builder);
	}

	/**
	 * Sets the attribute as property.
	 * 
	 * @param element
	 *            the element
	 * @param attribute
	 *            the attribute
	 * @param builder
	 *            the builder
	 */
	protected void setAttributeAsProperty(Element element, String attribute,
			BeanDefinitionBuilder builder) {
		setAttributeAsProperty(element, attribute, attribute, builder);
	}

	/**
	 * Sets the attribute as property.
	 * 
	 * @param element
	 *            the element
	 * @param attribute
	 *            the attribute
	 * @param property
	 *            the property
	 * @param builder
	 *            the builder
	 */
	protected void setAttributeAsProperty(Element element, String attribute,
			String property, BeanDefinitionBuilder builder) {
		if (element.hasAttribute(attribute)) {
			String value = element.getAttribute(attribute);
			builder.addPropertyValue(property, value);
		}
	}

	/**
	 * Sets the attribute as a reference.
	 * 
	 * @param element
	 *            the element
	 * @param attribute
	 *            the attribute
	 * @param property
	 *            the property
	 * @param builder
	 *            the builder
	 */
	protected void setAttributeAsReference(Element element, String attribute,
			String property, BeanDefinitionBuilder builder) {
		if (element.hasAttribute(attribute)) {
			String value = element.getAttribute(attribute);
			builder.addPropertyReference(property, value);
		}
	}

	/**
	 * Sets the elements as a list property.
	 * 
	 * @param element
	 *            the element
	 * @param name
	 *            the name of the child
	 * @param propertyName
	 *            the property that contains the list
	 * @param context
	 *            the context
	 * @param builder
	 *            the builder
	 */
	protected void setChildrenAsListProperty(Element element, String name,
			String propertyName, ParserContext context,
			BeanDefinitionBuilder builder) {
		List<Element> children = DOMUtils.getElements(element, name);
		if (!children.isEmpty()) {
			@SuppressWarnings("unchecked")
			List<BeanDefinition> list = new ManagedList(children.size());
			for (Element eChild : children) {
				BeanDefinition childDefinition = context
						.getDelegate()
						.parseCustomElement(eChild, builder.getBeanDefinition());
				list.add(childDefinition);
			}
			builder.addPropertyValue(propertyName, list);
		}
	}

	/**
	 * Sets the element as property.
	 * 
	 * @param element
	 *            the element
	 * @param name
	 *            the name
	 * @param context
	 *            the context
	 * @param builder
	 *            the builder
	 */
	protected void setElementAsProperty(Element element, String name,
			ParserContext context, BeanDefinitionBuilder builder) {
		setElementAsProperty(element, name, name, context, builder);
	}

	/**
	 * Sets the element as property.
	 * 
	 * @param element
	 *            the element
	 * @param name
	 *            the name
	 * @param propertyName
	 *            the property name
	 * @param context
	 *            the context
	 * @param builder
	 *            the builder
	 */
	protected void setElementAsProperty(Element element, String name,
			String propertyName, ParserContext context,
			BeanDefinitionBuilder builder) {
		Element eChild = DOMUtils.getElement(element, name);
		if (eChild != null) {
			BeanDefinition childDefinition = context.getDelegate()
					.parseCustomElement(eChild, builder.getBeanDefinition());
			context.getDelegate()
					.parsePropertyElements(eChild, childDefinition);
			builder.addPropertyValue(propertyName, childDefinition);
		}
	}

	/**
	 * Sets the list as property.
	 * 
	 * @param element
	 *            the element
	 * @param name
	 *            the name
	 * @param propertyName
	 *            the property name
	 * @param context
	 *            the context
	 * @param builder
	 *            the builder
	 */
	protected void setListAsProperty(Element element, String name,
			String propertyName, ParserContext context,
			BeanDefinitionBuilder builder) {
		Element eChild;
		if (StringUtils.isBlank(name)) {
			eChild = element;
		} else {
			eChild = DOMUtils.getElement(element, name);
		}
		if (eChild != null) {
			BeanDefinitionParserDelegate delegate = context.getDelegate();
			List<?> listDefinition = delegate.parseListElement(eChild, builder
					.getBeanDefinition());
			builder.addPropertyValue(propertyName, listDefinition);
		}
	}

	/**
	 * Sets the list of String as property.
	 * 
	 * @param element
	 *            the element
	 * @param name
	 *            the name
	 * @param propertyName
	 *            the property name
	 * @param context
	 *            the context
	 * @param builder
	 *            the builder
	 */
	protected void setListStringAsProperty(Element element, String name,
			String propertyName, ParserContext context,
			BeanDefinitionBuilder builder) {
		@SuppressWarnings("unchecked")
		List<String> stringList = new ManagedList();
		List<Element> eStringList = DOMUtils.getElements(element, name);
		for (Element eString : eStringList) {
			String string = DOMUtils.getText(eString);
			string = StringUtils.trim(string);
			stringList.add(string);
		}
		builder.addPropertyValue(propertyName, stringList);
	}

	/**
	 * Sets the map as property.
	 * 
	 * @param element
	 *            the element
	 * @param name
	 *            the name
	 * @param propertyName
	 *            the property name
	 * @param context
	 *            the context
	 * @param builder
	 *            the builder
	 */
	protected void setMapAsProperty(Element element, String name,
			String propertyName, ParserContext context,
			BeanDefinitionBuilder builder) {
		Element eChild;
		if (StringUtils.isBlank(name)) {
			eChild = element;
		} else {
			eChild = DOMUtils.getElement(element, name);
		}
		if (eChild != null) {
			BeanDefinitionParserDelegate delegate = context.getDelegate();
			Map<?, ?> mapDefinition = delegate.parseMapElement(eChild, builder
					.getBeanDefinition());
			builder.addPropertyValue(propertyName, mapDefinition);
		}
	}

	/**
	 * Sets a property value
	 * 
	 * @param element
	 *            the element
	 * @param propertyName
	 *            the property name
	 * @param propertyValue
	 *            the property value
	 * @param builder
	 *            the builder
	 */
	protected void setPropertyValue(Element element, String propertyName,
			String propertyValue, BeanDefinitionBuilder builder) {
		builder.addPropertyValue(propertyName, propertyValue);
	}
}
