package net.sf.doolin.context.support;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import org.springframework.beans.BeansException;
import org.springframework.beans.FatalBeanException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.BeanFactoryUtils;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.ListableBeanFactory;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.StringUtils;

public abstract class AbstractServiceNameFactoryBean<F> implements FactoryBean,
		BeanFactoryAware, InitializingBean {

	private Class<F> serviceInterface;
	private ListableBeanFactory beanFactory;
	private Object proxy;

	@Override
	public void afterPropertiesSet() throws Exception {
		// Create service locator proxy.
		this.proxy = Proxy.newProxyInstance(this.serviceInterface
				.getClassLoader(), new Class[] { this.serviceInterface },
				new InvocationHandler() {

					private Class<?> getServiceLocatorMethodReturnType(
							Method method) throws NoSuchMethodException {
						Class<?>[] paramTypes = method.getParameterTypes();
						Method interfaceMethod = AbstractServiceNameFactoryBean.this.serviceInterface
								.getMethod(method.getName(), paramTypes);
						Class<?> serviceLocatorReturnType = interfaceMethod
								.getReturnType();

						// Check whether the method is a valid service locator.
						if (paramTypes.length > 1
								|| void.class.equals(serviceLocatorReturnType)) {
							throw new UnsupportedOperationException(
									"May only call methods with signature '<type> xxx()' or '<type> xxx(<idtype> id)' "
											+ "on factory interface, but tried to call: "
											+ interfaceMethod);
						}
						return serviceLocatorReturnType;
					}

					public Object invoke(Object proxy, Method method,
							Object[] args) throws Throwable {
						Object service;
						if (Object.class.equals(method.getDeclaringClass())) {
							service = invokeStandardObjectMethod(method, args);
						} else {
							service = invokeServiceLocatorMethod(method, args);
						}
						return service;
					}

					private Object invokeServiceLocatorMethod(Method method,
							Object[] args) throws Exception {
						Class<?> serviceLocatorMethodReturnType = getServiceLocatorMethodReturnType(method);
						String beanName = tryGetBeanName(args);
						if (StringUtils.hasLength(beanName)) {
							// Service locator for a specific bean name.
							return AbstractServiceNameFactoryBean.this.beanFactory
									.getBean(beanName,
											serviceLocatorMethodReturnType);
						} else {
							// Service locator for a bean type.
							return BeanFactoryUtils
									.beanOfTypeIncludingAncestors(
											AbstractServiceNameFactoryBean.this.beanFactory,
											serviceLocatorMethodReturnType);
						}
					}

					/**
					 * It's normal to get here for non service locator interface
					 * method calls (toString, equals, etc.). Simply apply the
					 * call to the invocation handler object.
					 */
					private Object invokeStandardObjectMethod(Method method,
							Object[] args) throws Throwable {
						try {
							return method.invoke(this, args);
						} catch (InvocationTargetException invEx) {
							throw invEx.getTargetException();
						}
					}

					/**
					 * Check whether a service id was passed in.
					 */
					private String tryGetBeanName(Object[] args) {
						String beanName = "";
						if (args != null && args.length == 1 && args[0] != null) {
							beanName = args[0].toString();
						}
						return createBeanName(beanName);
					}
				});
	}

	protected abstract String createBeanName(String beanName);

	@Override
	public Object getObject() throws Exception {
		return this.proxy;
	}

	@Override
	public Class<F> getObjectType() {
		return this.serviceInterface;
	}

	public Class<F> getServiceInterface() {
		return this.serviceInterface;
	}

	@Override
	public boolean isSingleton() {
		return true;
	}

	@Override
	public void setBeanFactory(BeanFactory beanFactory) throws BeansException {
		if (!(beanFactory instanceof ListableBeanFactory)) {
			throw new FatalBeanException(
					"ServiceLocatorFactoryBean needs to run in a BeanFactory that is a ListableBeanFactory");
		}
		this.beanFactory = (ListableBeanFactory) beanFactory;
	}

	@Required
	public void setServiceInterface(Class<F> serviceInterface) {
		this.serviceInterface = serviceInterface;
	}

}
